//
//  DragSectionTableViewController.swift
//  DragAndDrop
//
//  Created by Toshihiro Morimoto on 2017/09/10.
//  Copyright © 2017 Toshihiro Morimoto. All rights reserved.
//

import UIKit

class DragSectionTableViewController: UITableViewController {
    
    var images = [#imageLiteral(resourceName: "scene_1")]

    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.dropDelegate = self
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return images.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "DragSectionTableViewCell", for: indexPath) as! DragSectionTableViewCell
        cell.iconImageView.image = images[indexPath.row]
        return cell
    }

    // MARK: - Table view delegate
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return view.bounds.width
    }
}

extension DragSectionTableViewController: UITableViewDropDelegate {
    func tableView(_ tableView: UITableView, performDropWith coordinator: UITableViewDropCoordinator) {
        let indexPath: IndexPath = coordinator.destinationIndexPath ??
            IndexPath(row: tableView.numberOfRows(inSection: 0), section: 0)
        coordinator.session.loadObjects(ofClass: UIImage.self) { images in
            guard let images = images as? [UIImage] else { return }

            var indexPaths = [IndexPath]()
            for (index, image) in images.enumerated() {
                self.images.append(image)
                let indexPath = IndexPath(row: indexPath.row + index, section: 0)
                indexPaths.append(indexPath)
            }
            tableView.insertRows(at: indexPaths, with: .automatic)
        }
    }
}

class DragSectionTableViewCell: UITableViewCell {
    @IBOutlet weak var iconImageView: UIImageView!
}
