//
//  ViewController.swift
//  ARKitSample
//
//  Created by Toshihiro Morimoto on 2017/09/10.
//  Copyright © 2017 Toshihiro Morimoto. All rights reserved.
//

import UIKit
import SceneKit
import ARKit

class ViewController: UIViewController {
    
    @IBOutlet var sceneView: ARSCNView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Set the view's delegate
        sceneView.delegate = self
        
        // Show statistics such as fps and timing information
        sceneView.showsStatistics = true
        
        // Create a new scene
        let scene = SCNScene(named: "art.scnassets/ship.scn")!
        
        // Set the scene to the view
        sceneView.scene = scene
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        // Create a session configuration
        let configuration = ARWorldTrackingConfiguration()
        
        // Run the view's session
        sceneView.session.run(configuration)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        // Pause the view's session
        sceneView.session.pause()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Release any cached data, images, etc that aren't in use.
    }
    
    @IBAction func sceneViewTapped(sender: UITapGestureRecognizer) {
        let results = sceneView.hitTest(sender.location(in: sceneView), types: [.featurePoint])
        guard let result = results.last else { return }
        
        // ノード（飛行機）を配置する位置を算出
        let transform = SCNMatrix4(result.worldTransform)
        let position = SCNVector3Make(transform.m41, transform.m42, transform.m43)
        
        // ノード（飛行機）を配置
        let scene = SCNScene(named: "art.scnassets/ship.scn")!
        let node = scene.rootNode
        node.position = position
        sceneView.scene.rootNode.addChildNode(node)
    }
}

extension ViewController: ARSCNViewDelegate {
    /*
     // Override to create and configure nodes for anchors added to the view's session.
     func renderer(_ renderer: SCNSceneRenderer, nodeFor anchor: ARAnchor) -> SCNNode? {
     let node = SCNNode()
     
     return node
     }
     */
    
    func session(_ session: ARSession, didFailWithError error: Error) {
        // Present an error message to the user
        
    }
    
    func sessionWasInterrupted(_ session: ARSession) {
        // Inform the user that the session has been interrupted, for example, by presenting an overlay
        
    }
    
    func sessionInterruptionEnded(_ session: ARSession) {
        // Reset tracking and/or remove existing anchors if consistent tracking is required
        
    }
}

