//
//  CoreMLViewController.swift
//  MachineLearningSample
//
//  Created by Toshihiro Morimoto on 2017/10/01.
//  Copyright © 2017 Toshihiro Morimoto. All rights reserved.
//

import UIKit
import CoreML

class CoreMLViewController: UIViewController {
    @IBOutlet weak var imageView: UIImageView!
    @IBOutlet weak var outputTextView: UITextView!
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        outputTextView.text = "now processing..."
    }

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        let image = makeImage()
        let pixelBuffer = convertBuffer(from: image)!
        let model = MobileNet()
        do {
            let output = try model.prediction(image: pixelBuffer)
            outputTextView.text = [
                "[classLabel]",
                output.classLabel,
                "",
                "[classLabelProbs]",
                output.classLabelProbs.description
            ].joined(separator: "\n")
        }
        catch let error as NSError {
            outputTextView.text = error.localizedDescription
        }
    }
    
    private func makeImage() -> UIImage {
        let image = imageView.image!

        // MobileNetに渡す画像が224px以内である必要があるため変換
        let size = CGSize(width: 224, height: 224)
        UIGraphicsBeginImageContext(size)
        defer {
            UIGraphicsEndImageContext()
        }
        image.draw(in: CGRect(origin: .zero, size: size))
        return UIGraphicsGetImageFromCurrentImageContext()!
    }
    
    private func convertBuffer(from image: UIImage) -> CVPixelBuffer? {
        let attrs = [
            kCVPixelBufferCGImageCompatibilityKey: kCFBooleanTrue,
            kCVPixelBufferCGBitmapContextCompatibilityKey: kCFBooleanTrue,
        ] as CFDictionary
        var pixelBuffer : CVPixelBuffer?
        let status = CVPixelBufferCreate(kCFAllocatorDefault, Int(image.size.width), Int(image.size.height), kCVPixelFormatType_32ARGB, attrs, &pixelBuffer)
        guard status == kCVReturnSuccess else {
            return nil
        }
        
        CVPixelBufferLockBaseAddress(pixelBuffer!, CVPixelBufferLockFlags(rawValue: 0))
        let pixelData = CVPixelBufferGetBaseAddress(pixelBuffer!)
        
        let rgbColorSpace = CGColorSpaceCreateDeviceRGB()
        let context = CGContext(
            data: pixelData,
            width: Int(image.size.width),
            height: Int(image.size.height),
            bitsPerComponent: 8,
            bytesPerRow: CVPixelBufferGetBytesPerRow(pixelBuffer!),
            space: rgbColorSpace,
            bitmapInfo: CGImageAlphaInfo.noneSkipFirst.rawValue)
        
        context?.translateBy(x: 0, y: image.size.height)
        context?.scaleBy(x: 1.0, y: -1.0)
        
        UIGraphicsPushContext(context!)
        image.draw(in: CGRect(x: 0, y: 0, width: image.size.width, height: image.size.height))
        UIGraphicsPopContext()
        CVPixelBufferUnlockBaseAddress(pixelBuffer!, .readOnly)
        
        return pixelBuffer
    }
}
